==> fap-0.8.1.tar.bz2: fap-0.8.1/src/audio/output/wmmPlugin/wmm.cpp <==
/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <wmm.h>

#ifdef _WIN32

const char wmm::name[]="WMM";

/*****************************************************************/

void CALLBACK wmm::waveOutProc(HWAVEOUT waveOut, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2) {

	if (uMsg==MM_WOM_DONE)
		SetEvent(*(HANDLE*)((WAVEHDR*)dwParam1)->dwUser);
}

/*****************************************************************/

wmm::wmm() :
	outputPlugin(name) {

	// Check devices
	WAVEOUTCAPS woc;
	const int n=waveOutGetNumDevs();

	for (int i=0; i<n; i++) {
		if (waveOutGetDevCaps(i, &woc, sizeof(WAVEOUTCAPS))==MMSYSERR_NOERROR)
#ifdef UNICODE
		{
			const FXint n=utfslen(woc.szPname);
			FXchar *buffer=new FXchar[n+1];
			nc2utfs(buffer, woc.szPname);
			addDevice(buffer);
			delete [] buffer;
		}
#else
			addDevice(woc.szPname);
#endif
	}
}

const size_t wmm::open(const unsigned int card, unsigned int &sampleRate, const unsigned int channels, const unsigned int prec) {

	WAVEFORMATEX wfm;

	memset(&wfm, 0, sizeof(WAVEFORMATEX));

	wfm.wFormatTag=WAVE_FORMAT_PCM;
	wfm.nSamplesPerSec=sampleRate;
	wfm.nChannels=channels;
	wfm.wBitsPerSample=prec<<3;
	wfm.nBlockAlign=channels*prec;
	wfm.nAvgBytesPerSec=wfm.nSamplesPerSec*wfm.nBlockAlign;
	//wfm.cbSize=0;

	MMRESULT err=waveOutOpen(&_wHandle, card, &wfm, (DWORD)waveOutProc, 0, CALLBACK_FUNCTION);
	if (err!=MMSYSERR_NOERROR) {
		fxmessage("Error opening device %s\n%s\n", device(card).text(), checkErr(err));
		return 0;
	}

	// Set buffer to contain 1/4 second of data
	const int bufferSize=wfm.nAvgBytesPerSec>>2;
	FXTRACE((10, "blocksize=%d\n", bufferSize));

	_event[0]=CreateEvent(0, FALSE, FALSE, 0);
	_event[1]=CreateEvent(0, FALSE, FALSE, 0);

	memset(&_wHdr[0], 0, sizeof(WAVEHDR));
	_wHdr[0].lpData=new CHAR[bufferSize<<1];
	_wHdr[0].dwUser=(DWORD)&_event[0];

	memset(&_wHdr[1], 0, sizeof(WAVEHDR));
	_wHdr[1].lpData=_wHdr[0].lpData+bufferSize;
	_wHdr[1].dwUser=(DWORD)&_event[1];

	SetEvent(_event[1]);

	_idx=0;

	return bufferSize;
}

void wmm::close() {

	CloseHandle(_event[0]);
	CloseHandle(_event[1]);

	while (waveOutUnprepareHeader(_wHandle, &_wHdr[1-_idx], sizeof(WAVEHDR))!=MMSYSERR_NOERROR)
		fxsleep(10);

	waveOutClose(_wHandle);
	delete [] _wHdr[0].lpData;
}

const char* wmm::checkErr(const MMRESULT err) {

	switch (err) {
		default:
		case MMSYSERR_ERROR: return "Unspecified error";
		case MMSYSERR_BADDEVICEID: return "Device ID out of range";
		case MMSYSERR_NOTENABLED: return "Driver failed enable";
		case MMSYSERR_ALLOCATED: return "Device already allocated";
		case MMSYSERR_INVALHANDLE: return "Device handle is invalid";
		case MMSYSERR_NODRIVER: return "No device driver present";
		case MMSYSERR_NOMEM: return "Memory allocation error";
		case MMSYSERR_NOTSUPPORTED: return "Function isn't supported";
		case MMSYSERR_BADERRNUM: return "Error value out of range";
		case MMSYSERR_INVALFLAG: return "Invalid flag passed";
		case MMSYSERR_INVALPARAM: return "Invalid parameter passed";
		case MMSYSERR_HANDLEBUSY: return "Handle being used simultaneously on another thread (eg callback)";
		case MMSYSERR_INVALIDALIAS: return "Specified alias not found";
		case MMSYSERR_BADDB: return "Bad registry database";
		case MMSYSERR_KEYNOTFOUND: return "Registry key not found";
		case MMSYSERR_READERROR: return "Registry read error";
		case MMSYSERR_WRITEERROR: return "Registry write error";
		case MMSYSERR_DELETEERROR: return "Registry delete error";
		case MMSYSERR_VALNOTFOUND: return "Registry value not found";
		case MMSYSERR_NODRIVERCB: return "Driver does not call DriverCallback";
	}
}

bool wmm::write(void* buffer, size_t bufferSize) {

	_wHdr[_idx].dwBufferLength=bufferSize;
	_wHdr[_idx].dwFlags=0;
	
	MMRESULT err;

	err=waveOutPrepareHeader(_wHandle, &_wHdr[_idx], sizeof(WAVEHDR));
	if (err!=MMSYSERR_NOERROR) {
		fxmessage("waveOutPrepareHeader: %s\n", checkErr(err));
		return false;
	}

	ResetEvent(_event[_idx]);

	err=waveOutWrite(_wHandle, &_wHdr[_idx], sizeof(WAVEHDR));
	if (err!=MMSYSERR_NOERROR) {
		fxmessage("waveOutWrite: %s\n", checkErr(err));
		return false;
	}

	_idx=1-_idx;

	if (WaitForSingleObject(_event[_idx], INFINITE)!=WAIT_OBJECT_0) {
		fxmessage("Error waiting for sound to finish\n");
		return false;
	}

	err=waveOutUnprepareHeader(_wHandle, &_wHdr[_idx], sizeof(WAVEHDR));
	if (err!=MMSYSERR_NOERROR) {
		fxmessage("waveOutUnprepareHeader: %s\n", checkErr(err));
		return false;
	}

	return true;
}

void wmm::stop() {

	waveOutReset(_wHandle);
}

void wmm::volume(int vol) {

	vol=(vol*65535)/100;
	vol|=(vol<<16);
	waveOutSetVolume(_wHandle, vol);
}

int wmm::volume() {

	DWORD vol;
	waveOutGetVolume(_wHandle, &vol);
	return (int)(((vol&0xFFFF)*100)/65535);
}

#endif

==> fap_0.8.8.orig.tar.bz2: fap-0.8.8/src/audio/output/wmmPlugin/wmm.cpp <==
/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <wmm.h>

#ifdef _WIN32

const char wmm::name[]="WMM";

/*****************************************************************/

void CALLBACK wmm::waveOutProc(HWAVEOUT waveOut, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2) {

	if (uMsg==MM_WOM_DONE)
		SetEvent(*(HANDLE*)((WAVEHDR*)dwParam1)->dwUser);
}

/*****************************************************************/

wmm::wmm() :
	outputPlugin(name) {

	// Check devices
	WAVEOUTCAPS woc;
	const int n=waveOutGetNumDevs();

	for (int i=0; i<n; i++) {
		if (waveOutGetDevCaps(i, &woc, sizeof(WAVEOUTCAPS))==MMSYSERR_NOERROR)
#ifdef UNICODE
		{
			const FXint n=utfslen(woc.szPname);
			FXchar *buffer=new FXchar[n+1];
			nc2utfs(buffer, woc.szPname);
			addDevice(buffer);
			delete [] buffer;
		}
#else
			addDevice(woc.szPname);
#endif
	}
}

const size_t wmm::open(const unsigned int card, unsigned int &sampleRate, const unsigned int channels, const unsigned int prec) {

	WAVEFORMATEX wfm;

	memset(&wfm, 0, sizeof(WAVEFORMATEX));

	wfm.wFormatTag=WAVE_FORMAT_PCM;
	wfm.nSamplesPerSec=sampleRate;
	wfm.nChannels=channels;
	wfm.wBitsPerSample=prec<<3;
	wfm.nBlockAlign=channels*prec;
	wfm.nAvgBytesPerSec=wfm.nSamplesPerSec*wfm.nBlockAlign;
	//wfm.cbSize=0;

	MMRESULT err=waveOutOpen(&_wHandle, card, &wfm, (DWORD)waveOutProc, 0, CALLBACK_FUNCTION);
	if (err!=MMSYSERR_NOERROR) {
		fxmessage("Error opening device %s\n%s\n", device(card).text(), checkErr(err));
		return 0;
	}

	// Set buffer to contain 1 second of data
	const int bufferSize=wfm.nAvgBytesPerSec;
	FXTRACE((10, "blocksize=%d\n", bufferSize));

	_event[0]=CreateEvent(0, FALSE, FALSE, 0);
	_event[1]=CreateEvent(0, FALSE, FALSE, 0);

	memset(&_wHdr[0], 0, sizeof(WAVEHDR));
	_wHdr[0].lpData=new CHAR[bufferSize<<1];
	_wHdr[0].dwUser=(DWORD)&_event[0];

	memset(&_wHdr[1], 0, sizeof(WAVEHDR));
	_wHdr[1].lpData=_wHdr[0].lpData+bufferSize;
	_wHdr[1].dwUser=(DWORD)&_event[1];

	SetEvent(_event[1]);

	_idx=0;

	return bufferSize;
}

void wmm::close() {

	CloseHandle(_event[0]);
	CloseHandle(_event[1]);

	while (waveOutUnprepareHeader(_wHandle, &_wHdr[1-_idx], sizeof(WAVEHDR))!=MMSYSERR_NOERROR)
		fxsleep(10);

	waveOutClose(_wHandle);
	delete [] _wHdr[0].lpData;
}

const char* wmm::checkErr(const MMRESULT err) {

	switch (err) {
		default:
		case MMSYSERR_ERROR: return "Unspecified error";
		case MMSYSERR_BADDEVICEID: return "Device ID out of range";
		case MMSYSERR_NOTENABLED: return "Driver failed enable";
		case MMSYSERR_ALLOCATED: return "Device already allocated";
		case MMSYSERR_INVALHANDLE: return "Device handle is invalid";
		case MMSYSERR_NODRIVER: return "No device driver present";
		case MMSYSERR_NOMEM: return "Memory allocation error";
		case MMSYSERR_NOTSUPPORTED: return "Function isn't supported";
		case MMSYSERR_BADERRNUM: return "Error value out of range";
		case MMSYSERR_INVALFLAG: return "Invalid flag passed";
		case MMSYSERR_INVALPARAM: return "Invalid parameter passed";
		case MMSYSERR_HANDLEBUSY: return "Handle being used simultaneously on another thread (eg callback)";
		case MMSYSERR_INVALIDALIAS: return "Specified alias not found";
		case MMSYSERR_BADDB: return "Bad registry database";
		case MMSYSERR_KEYNOTFOUND: return "Registry key not found";
		case MMSYSERR_READERROR: return "Registry read error";
		case MMSYSERR_WRITEERROR: return "Registry write error";
		case MMSYSERR_DELETEERROR: return "Registry delete error";
		case MMSYSERR_VALNOTFOUND: return "Registry value not found";
		case MMSYSERR_NODRIVERCB: return "Driver does not call DriverCallback";
	}
}

bool wmm::write(void* buffer, size_t bufferSize) {

	_wHdr[_idx].dwBufferLength=bufferSize;
	_wHdr[_idx].dwFlags=0;
	
	MMRESULT err;

	err=waveOutPrepareHeader(_wHandle, &_wHdr[_idx], sizeof(WAVEHDR));
	if (err!=MMSYSERR_NOERROR) {
		fxmessage("waveOutPrepareHeader: %s\n", checkErr(err));
		return false;
	}

	ResetEvent(_event[_idx]);

	err=waveOutWrite(_wHandle, &_wHdr[_idx], sizeof(WAVEHDR));
	if (err!=MMSYSERR_NOERROR) {
		fxmessage("waveOutWrite: %s\n", checkErr(err));
		return false;
	}

	_idx=1-_idx;

	if (WaitForSingleObject(_event[_idx], INFINITE)!=WAIT_OBJECT_0) {
		fxmessage("Error waiting for sound to finish\n");
		return false;
	}

	err=waveOutUnprepareHeader(_wHandle, &_wHdr[_idx], sizeof(WAVEHDR));
	if (err!=MMSYSERR_NOERROR) {
		fxmessage("waveOutUnprepareHeader: %s\n", checkErr(err));
		return false;
	}

	return true;
}

void wmm::stop() {

	waveOutReset(_wHandle);
}

void wmm::volume(int vol) {

	vol=(vol*65535)/100;
	vol|=(vol<<16);
	waveOutSetVolume(_wHandle, vol);
}

int wmm::volume() {

	DWORD vol;
	waveOutGetVolume(_wHandle, &vol);
	return (int)(((vol&0xFFFF)*100)/65535);
}

#endif
